import numpy as np

from ase.calculators.calculator import Calculator
from ase.neighborlist import neighbor_list

from ase.constraints import FixAtoms, FixBondLengths

def get_unit_vector(vector):
	unit_vector = vector / np.linalg.norm(vector)
	return unit_vector

class coulomb_potential(Calculator):
	implemented_properties = ['energy', 'forces']
	# k from https://en.wikipedia.org/wiki/Coulomb_constant in units eV·Å·e−2
	default_parameters = {'k': 14.3996,'Q': 1.0,'q': 1.0,'rcut': 20.0}
	
	def __init__(self, **kwargs):
		Calculator.__init__(self, **kwargs)
		self.previous_energy = float('inf')

	def calculate(self, atoms=None, properties=['energy'],system_changes=['positions', 'numbers', 'cell', 'pbc', 'charges', 'magmoms']):
		Calculator.calculate(self, atoms, properties, system_changes)

		k = self.parameters.k
		Q = self.parameters.Q
		q = self.parameters.q
		rcut = self.parameters.rcut

		forces = np.zeros((len(self.atoms), 3))
		coefficient = k*Q*q

		ii, jj, dd, DD = neighbor_list('ijdD', atoms, rcut)
		dhat = (DD / dd[:, None]).T

		energy = (coefficient / dd).sum()
		F = -((coefficient / dd**2.0) * dhat).T
		for dim in range(3):
			forces[:, dim] = np.bincount(ii, weights=F[:, dim],minlength=len(atoms))
		'''
		# ------------------------------------------------------------
		# remove component of force that is parallel to the sum of vectors of FixBondLength, FixBondLengths
		bonding_pairs_indices = []
		for constraints in atoms.constraints:
			if isinstance(constraints,FixBondLengths):
				for pair in constraints.pairs:
					bonding_pairs_indices.append(tuple(pair))
		pointing_vector = np.array([0.0,0.0,0.0])
		bonding_pairs_indices = tuple(set(bonding_pairs_indices))
		for bonding_pair_indices in bonding_pairs_indices:
			bonding_pair_indices = sorted(bonding_pair_indices)
			vector = atoms[bonding_pair_indices[1]].position - atoms[bonding_pair_indices[0]].position
			pointing_vector += vector
		pointing_vector = get_unit_vector(pointing_vector)
		force_on_hydrogen = forces[-1, :]
		# get vector projection onto unit vector: https://en.wikipedia.org/wiki/Vector_projection
		project_vector = np.dot(force_on_hydrogen,pointing_vector)*pointing_vector
		forces[-1, :] -= project_vector
		#import pdb; pdb.set_trace()
		'''
		# ------------------------------------------------------------
		# turn off forces for atoms that are fixed in position
		'''
		fixed_atom_indices = []
		for constraints in atoms.constraints:
			if isinstance(constraints,FixAtoms):
				fixed_atom_indices += tuple(constraints.get_indices())
		fixed_atom_indices = tuple(set(fixed_atom_indices))
		for index in fixed_atom_indices:
			forces[index, :] = np.array([0.0, 0.0, 0.0])
		'''
		#print(forces)
		# ------------------------------------------------------------
		self.results['energy'] = energy
		self.results['forces'] = forces



def get_distance(atom1,atom2):
	xx_dist = atom1.x - atom2.x
	yy_dist = atom1.y - atom2.y
	zz_dist = atom1.z - atom2.z
	distance = (xx_dist**2.0 + yy_dist**2.0 + zz_dist**2.0)**0.5
	return distance

def get_neighbour_list_for_surface_atoms(cluster,surface_atoms,cutoff_dictionary):
	surface_atom_neighbour_list = {}
	for index1 in range(len(surface_atoms)):
		surface_atom_index_1 = surface_atoms[index1]
		atom1 = cluster[surface_atom_index_1]
		for index2 in range(index1+1,len(surface_atoms)):
			surface_atom_index_2 = surface_atoms[index2]
			atom2 = cluster[surface_atoms[index2]]
			distance = get_distance(atom1,atom2)
			if distance <= cutoff_dictionary[(atom1.symbol,atom2.symbol)]:
				surface_atom_neighbour_list.setdefault(surface_atom_index_1,[]).append(surface_atom_index_2)
				surface_atom_neighbour_list.setdefault(surface_atom_index_2,[]).append(surface_atom_index_1)
	surface_atom_neighbour_list_sorted = {}
	for index in surface_atom_neighbour_list.keys():
		surface_atom_neighbour_list_sorted[index] = sorted(surface_atom_neighbour_list[index])
	return surface_atom_neighbour_list_sorted


#!/usr/bin/env python3
'''
Geoffrey Weal, Run_Adsorber_submitSL_slurm.py, 16/06/2021

This program is designed to submit all sl files called submit.sl to slurm

'''
print('###########################################################################')
print('###########################################################################')
print('Run_submitSl_slurm.py')
print('###########################################################################')
print('This program is designed to submit all your submit.sl scripts appropriately to slurm.')
print('###########################################################################')
print('###########################################################################')

import os, time, sys
import subprocess

# ------------------------------------------------------
# These variables can be changed by the user.
Max_jobs_in_queue_at_any_one_time = 10000
time_to_wait_before_next_submission = 20.0
time_to_wait_max_queue = 60.0

time_to_wait_before_next_submission_due_to_temp_submission_issue = 10.0
number_of_consecutive_error_before_exitting = 20

time_to_wait_before_next_submission_due_to_not_waiting_between_submissions = 60.0
# ------------------------------------------------------

if len(sys.argv) > 1:
    wait_between_submissions = str(sys.argv[1]).lower()
    if wait_between_submissions in ['t','true']:
        wait_between_submissions = True
    elif wait_between_submissions in ['f','false']:
        wait_between_submissions = False
    else:
        print('If you pass this program an argument, it must be either: ')
        print('    t, true, True: will wait 1 minute between submitting jobs')
        print('    f, false. False: will not wait between submitting jobs')
        print('If no argument is entered, the default is given as True')
        exit('This program will exit without running')
else:
    wait_between_submissions = False

if wait_between_submissions == True:
    print('This program will wait one minute between submitting jobs.')
else:
    print('This program will not wait between submitting jobs.')
path = os.getcwd()

def countdown(t):
    print('Will wait for ' + str(float(t)/60.0) + ' minutes, then will resume Slurm submissions.\n')
    while t:
        mins, secs = divmod(t, 60)
        timeformat = str(mins) + ':' + str(secs)
        #timeformat = '{:02d}:{:02d}'.format(mins, secs)
        sys.stdout.write("\r                                                                                   ")
        sys.stdout.flush()
        sys.stdout.write("\rCountdown: " + str(timeformat))
        sys.stdout.flush()
        time.sleep(1)
        t -= 1
    print('Resuming Pan Submissions.\n')

def myrun(cmd):
    proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout_lines = list(iter(proc.stdout.readline,b''))
    stdout_lines = [line.decode() for line in stdout_lines]
    #stdout_lines = [print(line) for line in iter(proc.stdout.readline,'')]
    #import pdb; pdb.set_trace()
    #proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    #stdout_lines = [line for line in io.TextIOWrapper(proc.stdout, encoding="utf-8")]
    """from http://blog.kagesenshi.org/2008/02/teeing-python-subprocesspopen-output.html"""
    '''
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout = []
    while True:
        line = p.stdout.readline()
        stdout.append(line)
        if line == '' and p.poll() != None:
            break
    '''
    return ''.join(stdout_lines)

###########################################################################################
###########################################################################################
###########################################################################################

def get_number_to_trials_that_will_be_submitted_by_submitSL(dirpath):
    return 1

command = "squeue -r -u $USER"
def check_max_jobs_in_queue_after_next_submission(dirpath):
    while True:
        text = myrun(command)
        nlines = len(text.splitlines())-1
        if not (nlines == -1):
            break
        else:
            print('Could not get the number of jobs in the slurm queue. Retrying to get this value.')
    number_of_trials_to_be_submitted = get_number_to_trials_that_will_be_submitted_by_submitSL(dirpath)
    if nlines > Max_jobs_in_queue_at_any_one_time - number_of_trials_to_be_submitted:
        return True, nlines
    else:
        return False, nlines

###########################################################################################
###########################################################################################
###########################################################################################

# Time to submit all the GA scripts! Lets get this stuff going!
if not wait_between_submissions:
    max_consec_counter = 250
    consec_counter = 0
submitting_command = "sbatch submit.sl"
for (dirpath, dirnames, filenames) in os.walk(path):
    dirnames.sort()
    if 'submit.sl' in filenames:
        if 'OUTCAR' in filenames:
            dirnames[:] = []
            filenames[:] = []
            continue
        # determine if it is the right time to submit jobs
        print('*****************************************************************************')
        while True:
            reached_max_jobs, number_in_queue = check_max_jobs_in_queue_after_next_submission(dirpath)
            if reached_max_jobs:
                print('-----------------------------------------------------------------------------')
                print('You can not have any more jobs in the queue before submitting the mass_sub. Will wait a bit of time for some of them to complete')
                print('Number of Jobs in the queue = '+str(number_in_queue))
                countdown(time_to_wait_before_next_submission)
                print('-----------------------------------------------------------------------------')
            else:
                print('The number of jobs in the queue currently is: '+str(number_in_queue))
                break
        # submit the jobs
        os.chdir(dirpath)
        name = dirpath.replace(path, '').split('/', -1)[1:]
        name = "_".join(str(x) for x in name)
        print("Submitting " + str(name) + " to slurm.")
        print('Submission .sl file found in: '+str(os.getcwd()))
        error_counter = 0
        while True:
            if error_counter == number_of_consecutive_error_before_exitting:
                break
            else:
                proc = subprocess.Popen(submitting_command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                if not proc.wait() == 0:
                    error_counter += 1
                    if error_counter == number_of_consecutive_error_before_exitting:
                        print('----------------------------------------------')
                        print('Error in submitting submit script to slurm.')
                        print('I got '+str(number_of_consecutive_error_before_exitting)+" consecutive errors. Something must not be working right somewhere. I'm going to stop here just in case something is not working.")
                        print('')
                        print('The following submit.sl scripts WERE NOT SUBMITTED TO SLURM')
                        print('')
                    else:
                        stdout, stderr = proc.communicate()
                        print('----------------------------------------------')
                        print('Error in submitting submit script to slurm. This error was:')
                        print(stderr)
                        print('Number of consecutive errors: '+str(error_counter))
                        print('Run_submitSL_slurm.py will retry submitting this job to slurm after '+str(time_to_wait_before_next_submission_due_to_temp_submission_issue)+' seconds of wait time')
                        print('----------------------------------------------')
                        countdown(time_to_wait_before_next_submission_due_to_temp_submission_issue)
                else:
                    break
        if error_counter == number_of_consecutive_error_before_exitting:
            print(dirpath)
        else:
            if wait_between_submissions:
                reached_max_jobs, number_in_queue = check_max_jobs_in_queue_after_next_submission(dirpath)
                print('The number of jobs in the queue after submitting job is currently is: '+str(number_in_queue))
                #print('Will wait for '+str(time_to_wait_max_queue)+' to give time between consecutive submissions')
                countdown(time_to_wait_max_queue)
                print('*****************************************************************************')
        dirnames[:] = []
        filenames[:] = []
        if not wait_between_submissions:
            if consec_counter >= max_consec_counter:
                print('----------------------------------------------')
                print('As you are not waiting between consecutive submissions, it is good practise to wait for a minute at some stage')
                print(str(max_consec_counter) +' have been submitted consecutively. Will not wait for '+str(time_to_wait_before_next_submission_due_to_not_waiting_between_submissions)+' s before continuing')
                print('----------------------------------------------')
                countdown(time_to_wait_before_next_submission_due_to_not_waiting_between_submissions)
                consec_counter = 0
            else:
                consec_counter += 1


if error_counter == number_of_consecutive_error_before_exitting:
    print('----------------------------------------------')
    print()
    print('"Run_submitSL_slurm.py" will finish WITHOUT HAVING SUBMITTED ALL JOBS.')
    print()
    print('*****************************************************************************')
    print('NOT ALL submit.sl SCRIPTS WERE SUBMITTED SUCCESSFULLY.')
    print('*****************************************************************************') 
else:
    print('*****************************************************************************')
    print('*****************************************************************************')
    print('*****************************************************************************')
    print('All submit.sl scripts have been submitted to slurm successfully.')
    print('*****************************************************************************')
    print('*****************************************************************************')
    print('*****************************************************************************')


import numpy as np
from ase import Atom
from ase.optimize import FIRE
from ase.constraints import FixAtoms, FixBondLengths

from Adsorber.Adsorber.coulomb_potential import coulomb_potential
from Adsorber.Adsorber.other_methods import get_distance, get_unit_vector

def get_the_centre(atom_indices,cluster_positions):
	vectors = [cluster_positions[index] for index in atom_indices]
	centre_of_shape = sum(vectors)/float(len(atom_indices))
	return centre_of_shape

def coulomb_repulsion_potential_optimisation(cluster,binding_point,surface_atoms_to_bind_to):
	new_cluster = cluster.copy()
	# fix every atom in the cluster
	const1 = FixAtoms(indices=[atom.index for atom in new_cluster])
	# append a dummy adsorption atom and fix its bond distance with surface atoms
	new_cluster.append(Atom('H',binding_point))
	# fix bond length ot atoms of interest to bond to on surface
	bonds_to_fix = [(surface_atom,len(new_cluster)-1) for surface_atom in surface_atoms_to_bind_to]
	const2 = FixBondLengths(bonds_to_fix)
	# set constraints
	new_cluster.set_constraint([const1,const2])
	new_cluster.set_calculator(coulomb_potential())
	opt = FIRE(new_cluster) #,trajectory='qn.traj')
	opt.run(fmax=0.02,steps=100)
	binding_point = new_cluster[-1].position
	return binding_point

def get_origin_position(cluster_or_surface_model,cluster_center_of_mass,surface_atom_position):
	if cluster_or_surface_model == 'cluster':
		origin_position = cluster_center_of_mass
	elif cluster_or_surface_model == 'surface model':
		origin_position = np.copy(surface_atom_position)
		origin_position[2] = -9999999999999999.9 #-float('inf')
	return origin_position

# ------------------------------------------------------------------------------------------------------------------------------------ #
# Adatom above atom

def get_above_atom_adsorption_site(surface_atom,cluster,cluster_positions,distance_of_adatom_from_surface,origin_position):
	# get cluster positions
	surface_atom_position = cluster_positions[surface_atom]
	# get line from centre of mass of cluster to the surface atom of interest
	AB = surface_atom_position - origin_position
	AB_unit_vector = get_unit_vector(AB)
	#get new binding postion for the adsorbed atom
	binding_point = surface_atom_position + distance_of_adatom_from_surface*AB_unit_vector
	# to make sure it is as far away from every other atom as possible, perform a coulomb repulsion potential optimisation
	surface_atoms_to_bind_to = [surface_atom]
	binding_point = coulomb_repulsion_potential_optimisation(cluster,binding_point,surface_atoms_to_bind_to)
	return (binding_point,surface_atom_position)


def get_above_atom_sites(cluster,surface_atoms,distance_of_adatom_from_surface,cluster_or_surface_model):
	binding_point_data = []
	cluster_center_of_mass = cluster.get_center_of_mass()
	cluster_positions = cluster.get_positions()
	original_no_of_atoms = len(cluster)
	for surface_atom in surface_atoms:
		origin_position = get_origin_position(cluster_or_surface_model,cluster_center_of_mass,cluster_positions[surface_atom])
		binding_point_datum = get_above_atom_adsorption_site(surface_atom,cluster,cluster_positions,distance_of_adatom_from_surface,origin_position)
		binding_point_data.append(binding_point_datum)
	return binding_point_data

# ------------------------------------------------------------------------------------------------------------------------------------ #
# Adatom on bridging site

def get_above_bridge_adsorption_site(surface_atoms_to_bind_to,cluster,cluster_positions,distance_of_adatom_from_surface,origin_position):
	# get the centre of the bridge
	centre_of_pair = get_the_centre(surface_atoms_to_bind_to,cluster_positions)
	# get line from centre of mass of cluster to the surface atom of interest
	AB = centre_of_pair - origin_position
	AB_unit_vector = get_unit_vector(AB)
	# make a plane with the normal vector as the atom_to_atom_vector
	atom_to_atom_vector = [cluster_positions[index] for index in surface_atoms_to_bind_to]
	atom_to_atom_vector = atom_to_atom_vector[1] - atom_to_atom_vector[0]
	atom_to_atom_unit_vector = get_unit_vector(atom_to_atom_vector)
	# project the point from the AB_unit_vector onto a plane with normal vector atom_to_atom_unit_vector
	# This will give a point that is perp to atom_to_atom_unit_vector and is point away from the centre of mass of the cluster
	# https://stackoverflow.com/questions/9605556/how-to-project-a-point-onto-a-plane-in-3d
	orig = np.array([0.0,0.0,0.0]) # centre of unit vector plane is the origin.
	point = AB_unit_vector - orig
	normal = atom_to_atom_unit_vector
	distance = np.dot(point,normal)
	projected_point = point - distance*normal
	perp_vector_to_bond_unit_vector = get_unit_vector(projected_point)
	# get new binding postion for the adsorbed atom
	binding_point = centre_of_pair + distance_of_adatom_from_surface*perp_vector_to_bond_unit_vector
	# to make sure it is as far away from every other atom as possible, perform a coulomb repulsion potential optimisation
	binding_point = coulomb_repulsion_potential_optimisation(cluster,binding_point,surface_atoms_to_bind_to)
	return (binding_point,centre_of_pair)

def get_bridge_sites(cluster,neighbour_list_original,distance_of_adatom_from_surface,cluster_or_surface_model):
	neighbour_list = dict(neighbour_list_original)
	binding_point_data = []
	cluster_center_of_mass = cluster.get_center_of_mass()
	cluster_positions = cluster.get_positions()
	for surface_index1, indices2 in sorted(neighbour_list.items(),key=lambda x:x[0]):
		for surface_index2 in sorted(indices2):
			surface_atoms_to_bind_to = (surface_index1,surface_index2)
			centre_of_pair = get_the_centre(surface_atoms_to_bind_to,cluster_positions)
			origin_position = get_origin_position(cluster_or_surface_model,cluster_center_of_mass,centre_of_pair)
			binding_point_datum = get_above_bridge_adsorption_site(surface_atoms_to_bind_to,cluster,cluster_positions,distance_of_adatom_from_surface,origin_position)
			binding_point_data.append(binding_point_datum)
			neighbour_list[surface_index2].remove(surface_index1)
	return binding_point_data

# ------------------------------------------------------------------------------------------------------------------------------------ #
# Adatom on triangle site

def get_three_or_four_adsorption_site(shape_indices,cluster_positions,distance_of_adatom_from_surface,origin_position):

	# Find the centre of the triangle
	centre_of_shape = get_the_centre(shape_indices,cluster_positions)
	'''
	###### NOT USED ANYMORE ######
	# find distance to add the adatom to above the triangle to give a distance of distance_of_adatom_from_surface_atom from each atom. Assuming equillatoral triangle.
	distance_in_triangle = get_distance(centre_of_triangle,cluster_positions[triangle_indices[0]])
	distance_from_triangle = (distance_of_adatom_from_surface_atom**2.0 - distance_in_triangle**2.0)*0.5
	'''
	# Get the normal vector to the plane that the triangle is in
	AB = cluster_positions[shape_indices[1]] - cluster_positions[shape_indices[0]]
	AC = cluster_positions[shape_indices[2]] - cluster_positions[shape_indices[0]]
	normal_vector = np.cross(AB,AC)
	normal_unit_vector = get_unit_vector(normal_vector)
	# Determine the position of the adatom bonding site. Could be two positions, use centre of mass to use the point that is furest away.
	vector_to_add_to_centre_of_triangle = normal_unit_vector*distance_of_adatom_from_surface
	adsorption_site_1 = centre_of_shape + vector_to_add_to_centre_of_triangle
	adsorption_site_2 = centre_of_shape - vector_to_add_to_centre_of_triangle
	distance_from_COM_to_site_1 = get_distance(origin_position,adsorption_site_1)
	distance_from_COM_to_site_2 = get_distance(origin_position,adsorption_site_2)
	if distance_from_COM_to_site_1 > distance_from_COM_to_site_2:
		return (adsorption_site_1,centre_of_shape)
	else:
		return (adsorption_site_2,centre_of_shape)

def get_three_fold_sites(cluster,neighbour_list,distance_of_adatom_from_surface_atom,cluster_or_surface_model):
	# find neighbouring sets of triangles
	triangles = []
	for index1, indices2 in neighbour_list.items():
		for index2 in indices2:
			neighbours_of_1_as_set = set(indices2)
			neighbours_of_2_as_set = set(neighbour_list[index2])
			intersections = neighbours_of_1_as_set & neighbours_of_2_as_set
			for index3 in intersections:
				triangles.append(tuple(sorted([index1,index2,index3])))
	triangles = sorted(list(set(triangles)))
	# get binding points
	binding_point_data = []
	cluster_positions = cluster.get_positions()
	cluster_center_of_mass = cluster.get_center_of_mass()
	bottom_of_z_axis = np.array((0.0,0.0,-9999999999999999.9))
	for triangle_indices in triangles:
		origin_position = get_origin_position(cluster_or_surface_model,cluster_center_of_mass,bottom_of_z_axis)
		binding_point_datum = get_three_or_four_adsorption_site(triangle_indices,cluster_positions,distance_of_adatom_from_surface_atom,origin_position)
		binding_point_data.append(binding_point_datum)
	return binding_point_data

# ------------------------------------------------------------------------------------------------------------------------------------ #
# Adatom on square site

def get_four_fold_sites(cluster,neighbour_list,distance_of_adatom_from_surface_atom,cluster_or_surface_model):
	# find neighbouring sets of squares
	squares = []
	for index1, indices2 in sorted(neighbour_list.items(),key=lambda x:x[0]):
		for ii in range(len(indices2)):
			first_neighbour_to_index1 = indices2[ii]
			neighbours_of_2_as_set = set(neighbour_list[first_neighbour_to_index1])
			for jj in range(ii+1,len(indices2)):
				second_neighbour_to_index1 = indices2[ii]
				neighbours_of_3_as_set = set(neighbour_list[second_neighbour_to_index1])
				intersections = neighbours_of_2_as_set & neighbours_of_3_as_set
				for index4 in intersections:
					square = tuple(sorted([index1,first_neighbour_to_index1,index4,second_neighbour_to_index1]))
					if not len(set(square)) == len(square):
						continue
					position_1 = neighbour_list[index1]
					position_4 = neighbour_list[index4]
					distance14 = get_distance(position_1,position_4)
					position_2 = neighbour_list[first_neighbour_to_index1]
					position_3 = neighbour_list[second_neighbour_to_index1]
					distance23 = get_distance(position_2,position_3)
					if (distance14 <= distance_of_adatom_from_surface_atom*(2.0)) and (distance23 <= distance_of_adatom_from_surface_atom*(2.0)):
						squares.append()
	squares = sorted(list(set(squares)))
	# get binding points
	binding_point_data = []
	cluster_positions = cluster.get_positions()
	cluster_center_of_mass = cluster.get_center_of_mass()
	bottom_of_z_axis = np.array((0.0,0.0,9999999999999999.9))
	for square_indices in squares:
		origin_position = get_origin_position(cluster_or_surface_model,cluster_center_of_mass,bottom_of_z_axis)
		binding_point_datum = get_three_or_four_adsorption_site(square_indices,cluster_positions,distance_of_adatom_from_surface_atom,origin_position)
		binding_point_data.append(binding_point_datum)
	return binding_point_data

# ------------------------------------------------------------------------------------------------------------------------------------ #



import numpy as np
from math import sqrt, cos, radians, atan, tan, atan2, degrees, acos, pi
from ase import Atom, Atoms
from ase.io import read, write
from ase.visualize import view

def get_unit_vector(non_normalised_vector):
	return non_normalised_vector/np.linalg.norm(non_normalised_vector)

name = '15-3-3629.xyz'
cluster = read(name)
cluster_positions = cluster.get_positions()
# Center center of mass of the cluster at the origin
cluster_center_of_mass = cluster.get_center_of_mass()
# Calculate the radial vectors of all of the atoms in the cluster, from the center --> call this vector AC
cluster_radial_vectors = cluster_positions - cluster_center_of_mass
# Calculate the vector between the atom of interest (atom i) and all other atoms in the system. --> call this vector AB
# Then dot product this with the radial vector for the atom of interest (atom i)
angle_of_cones_between_vectors = {}
for index1 in range(len(cluster_positions)):
	cluster_radial_vector_1_unit_vector = get_unit_vector(cluster_radial_vectors[index1])
	position1 = cluster_positions[index1]
	for index2 in range(index1+1,len(cluster_positions)):
		position2 = cluster_positions[index2]
		vector_from_1_to_2 = position2 - position1 # getting AB
		## take dot product of the vector AB and the radial vector 
		angle_of_cone = np.dot(get_unit_vector(vector_from_1_to_2),cluster_radial_vector_1_unit_vector)
		angle_of_cones_between_vectors.setdefault(index1,[]).append((index2,angle_of_cone))
		angle_of_cones_between_vectors.setdefault(index2,[]).append((index1,angle_of_cone))
#Check to see whether there are any other vectors (all vectors AB) that sit within a (in this case, currently, 40 degree cone). 
surface_atoms = []
for index1, cones in angle_of_cones_between_vectors.items():
	max_index, max_cone = max(cones,key=lambda x:x[1])
	if max_cone < cos(radians(40)): 
		cluster[index1].tag = 1
		print(index1)
	else:
		cluster[index1].tag = 0

write('surface_'+name,cluster)











# import numpy as np
# from math import sqrt, cos, radians, atan, tan, atan2, degrees, acos, pi

# nAt = 110

# header = []

# num_lines = sum(1 for line in open('XDAT_1'))
# iter = (num_lines-7)/(nAt+1)

# all_atoms = []
# # Read in the coordinates from the XDAT_1 file
# with open("XDAT_1", "r") as ofile:
# 	for i in range(7):
# 		x = ofile.readline()
# 		header.append(x)
# 	for j in range(iter):
# 		ind_atoms = []
# 		for k in range(nAt+1):
# 			x = ofile.readline()
# 			if not "Direct" in x:
# 				geometry_coords = []
# 				each_atom_coords = x.split()
# 				for l in range(3):
# 					geometry_coords.append(float(each_atom_coords[l])*40)
# 				ind_atoms.append(geometry_coords)
# 		all_atoms.append(ind_atoms)			

# center = []	

# #Center center of mass of the cluster at the origin

# for k in all_atoms:
# 	com = np.mean(k, axis = 0)
# 	center.append(com)

# #Calculate the radial vectors of all of the atoms in the cluster, from the center --> call this vector AC
	
# radial_vectors = []

# for k in range(len(all_atoms)):
# 	rv_iter = []
# 	for b in range(len(all_atoms[k])):
# 		rv_atom = []
# 		for i in range(3):
# 			rv = (all_atoms[k][b][i] - center[k][i])
# 			rv_atom.append(rv)
# 		rv_iter.append(rv_atom)
# 	radial_vectors.append(rv_iter)

# ab_v = []

# fw = open('chk.xyz', 'w')

# #Calculate the vector between the atom of interest (atom i) and all other atoms in the system. --> call this vector AB

# master_indices = []
# surface_atoms = []
# for m in range(len(all_atoms)):
# 	all_indices = []
# 	surf_at = []
# 	for a in range(len(all_atoms[m])):
# 		atom_index = []
# 		ab_v_atoms = []
# 		for b in range(nAt):
# 			ab_v = []	
# 			if b != a:
# 				for l in range(3):
# 					ab_vector = all_atoms[m][b][l] - all_atoms[m][a][l] ## calculate length of vector ab
# 					ab_v.append(ab_vector)
#                                 ## This part is to calculate the angle between vector AB and vector AC
# 				numerator = ((ab_v[0])*(radial_vectors[m][a][0]) + (ab_v[1])*(radial_vectors[m][a][1]) + (ab_v[2])*(radial_vectors[m][a][2])) ## take dot product of the vector AB and the radial vector 
# 				denominator = ((((ab_v[0])**2 + (ab_v[1])**2 + (ab_v[2])**2) ** 0.5) * (((radial_vectors[m][a][0])**2 + (radial_vectors[m][a][1])**2 + (radial_vectors[m][a][2])**2) ** 0.5)) ## magnitude of vector AB * magnitude of vector AC 
# 				index = numerator/denominator # index = cos(theta) 
# 				atom_index.append(index)
# 		all_indices.append(atom_index)
# 	master_indices.append(all_indices)

# #Check to see whether there are any other vectors (all vectors AB) that sit within a (in this case, currently, 40 degree cone). 

# for h in range(len(master_indices)):
# 	fw.write(str(nAt) +  '\nblah \n')
# 	for y in range(len(master_indices[h])):
# 		for p in master_indices[h][y]: ## You can change the angle of the cone that look for atoms within, to make the algorithm more or less selective. Bigger angle = more selective. 
# 		#	if max(master_indices[h][y]) < 0.5: #cos 60
# 	#		if max(master_indices[h][y]) < ((sqrt(2))/2): #cos 45
# 			#if max(master_indices[h][y]) < ((sqrt(3))/2): #cos 30
# 			if max(master_indices[h][y]) < cos(radians(40)): #cos 40
# 				lbl = 'Bi'
# 			else:
# 				lbl = 'Ga'
# 				print(h, y)
# 		outstr = '%s   %10.5f   %10.5f  %10.5f\n' % (lbl, all_atoms[h][y][0], all_atoms[h][y][1], all_atoms[h][y][2])
# 		fw.write(outstr)
 



import os
import numpy as np

from ase import Atom, Atoms
from ase.io import write

from Adsorber.Adsorber.import_settings_into_adsorber import import_settings
from Adsorber.Adsorber.neighbour_list import get_neighbour_list_for_surface_atoms
from Adsorber.Adsorber.get_places_to_bind_to import get_above_atom_sites, get_bridge_sites, get_three_fold_sites, get_four_fold_sites
from Adsorber.Adsorber.adsorb_single_species_to_cluster import adsorb_single_species_to_cluster
from Adsorber.Adsorber.storage_file import write_data_file, load_data_file
from Adsorber.Adsorber.Create_VASP_Files import make_VASP_folders
from Adsorber import __version__

def get_version_number():
	version = __version__
	return version

def version_no():
	"""
	Will provide the version of the Adsorber program
	"""
	version = get_version_number() 
	return version

class Adsorber_Program:

	def __init__(self,name,cluster_or_surface_model,cutoff,surface_atoms,adsorbed_species,slurm_information={}):
		import_settings(self,name,cluster_or_surface_model,cutoff,surface_atoms,adsorbed_species,slurm_information)
		self.introductory_remarks()
		#self.check_for_cluster_folder(self.system_folder_name)
		self.run()

	# ------------------------------------------------------------------------------------------------------------------------------------------------------

	def introductory_remarks(self):
		print('=================================')
		print()
		print('            Adsorber             ')
		print()
		print('         Version: '+str(version_no()))
		print()
		print('=================================')
		print()
		print()
		print()

	'''
	def check_for_cluster_folder(self,system_folder_name):
		print('=========================================================================================================')
		print('You already have a folder called '+str(system_folder_name))
		print('As it is likely you may remove some of the models from this folder as you decide which models to sample, Adsorber will not run any further.')
		print('If you want to run Adsorber, change the current '+str(system_folder_name)+' folder to a different name.')
		print('Adsorber will finish without starting.')
		print('=========================================================================================================')
		exit()
	'''

	# ------------------------------------------------------------------------------------------------------------------------------------------------------

	def run(self):
		self.set_up_cluster(self.cluster,self.surface_atoms,self.name_without_suffix)
		self.save_adsorbates(self.adsorbed_species)
		if not os.path.exists(self.system_folder_name):
			print('=================================')
			print('Adsorber will create systems with adsorbed atoms and molecules.')
			print('=================================')
			self.run_first_Adsorber_run()
		else:
			print('=================================')
			print('Adsorber has already created systems with adsorbed atoms and molecules.')
			print('Adsorber will attempt to create VASP directories of systems with adsorbed molecules from the '+str(self.system_folder_name)+' folder')
			print('=================================')
			self.make_VASP_files()
			print('=================================')

	def make_VASP_files(self):
		if not self.slurm_information == {}:
			make_VASP_folders(self.cluster,self.adsorbed_species,look_through_folder=self.system_folder_name,vasp_files_folder=self.vasp_files_folder,folder_name=self.VASP_folder_name,slurm_information=self.slurm_information)
		else:
			print('No information give for slurm_information')
			print('VASP files of systems with adsorbed molecules will not be created until a information about the submit.sl files are given in the slurm_information variable.')
			print('Adsorber will finish without making VASP files of systems with adsorbed molecules.')
			exit()

	def run_first_Adsorber_run(self):
		above_atom_binding_sites, above_bridge_binding_sites, above_three_fold_sites, above_four_fold_sites = self.get_initial_adatom_placements(self.cluster,self.cutoff,self.distance_of_adatom_from_surface,self.surface_atoms)
		print('============================================================================================')
		print('Make Clusters/Surfaces with adatoms/admolecules attached above atom sites')
		main_path_name = self.system_folder_name+'/Above_Atom_Sites'
		self.adsorb_species_to_cluster(self.cluster,above_atom_binding_sites,self.adsorbed_species,main_path_name)
		print('Make Clusters/Surfaces with adatoms/admolecules attached above bridge sites')
		main_path_name = self.system_folder_name+'/Above_Bridge_Sites'
		self.adsorb_species_to_cluster(self.cluster,above_bridge_binding_sites,self.adsorbed_species,main_path_name)
		print('Make Clusters/Surfaces with adatoms/admolecules attached above three-fold sites')
		main_path_name = self.system_folder_name+'/Above_Three_Fold_Sites'
		self.adsorb_species_to_cluster(self.cluster,above_three_fold_sites,self.adsorbed_species,main_path_name)
		print('Make Clusters/Surfaces with adatoms/admolecules attached above four-fold sites')
		main_path_name = self.system_folder_name+'/Above_Four_Fold_Sites'
		self.adsorb_species_to_cluster(self.cluster,above_four_fold_sites,self.adsorbed_species,main_path_name)
		print('============================================================================================')

	def set_up_cluster(self,cluster,surface_atoms,name_without_suffix):
		cluster.set_tags(0)
		for surface_atom_index in surface_atoms:
			cluster[surface_atom_index].tag = 1
		write(name_without_suffix+'_tagged_surface_atoms.xyz',cluster)

	def save_adsorbates(self,adsorbed_species):
		adsorbates = []
		for an_adsorbed_species in adsorbed_species:
			adsorbate = an_adsorbed_species['molecule'].copy()
			adsorbates.append(adsorbate)
		write('adsorbates.traj',adsorbates)

	def get_initial_adatom_placements(self,cluster,cutoff,distance_of_adatom_from_surface,surface_atoms):
		if not os.path.exists(self.data_storage_file):
			print('============================================================================================')
			print('Getting Binding data')
			neighbour_list = get_neighbour_list_for_surface_atoms(cluster,surface_atoms,cutoff)
			print('Getting above atoms binding sites.')
			above_atom_binding_sites = get_above_atom_sites(cluster,surface_atoms,distance_of_adatom_from_surface,self.cluster_or_surface_model)
			print('Getting above bridge binding sites.')
			above_bridge_binding_sites = get_bridge_sites(cluster,neighbour_list,distance_of_adatom_from_surface,self.cluster_or_surface_model)
			print('Getting above three-fold binding sites.')
			above_three_fold_sites = get_three_fold_sites(cluster,neighbour_list,distance_of_adatom_from_surface,self.cluster_or_surface_model)
			print('Getting above four-fold binding sites.')
			above_four_fold_sites = get_four_fold_sites(cluster,neighbour_list,distance_of_adatom_from_surface,self.cluster_or_surface_model)
			print('Saving data to '+str(self.data_storage_file))
			write_data_file(self.data_storage_file,self.bind_site_data_types,above_atom_binding_sites,above_bridge_binding_sites,above_three_fold_sites,above_four_fold_sites)
			print('============================================================================================')
		else:
			print('============================================================================================')
			print('Loading data from '+str(self.data_storage_file))
			above_atom_binding_sites, above_bridge_binding_sites, above_three_fold_sites, above_four_fold_sites = load_data_file(self.data_storage_file,self.bind_site_data_types)
			print('============================================================================================')
		def make_full_binding_site_representative_cluster(cluster,above_atom_binding_sites,cluster_file_name):
			representative_cluster = cluster.copy()
			counter = 1
			for binding_position, surface_position in above_atom_binding_sites:
				binding_representative_atom = Atom('H',position=binding_position,charge=counter,tag=2)
				representative_cluster.append(binding_representative_atom)
				counter += 1
			write(cluster_file_name+'.xyz',representative_cluster)
		make_full_binding_site_representative_cluster(cluster,above_atom_binding_sites,self.name_without_suffix  +'_above_atom_binding_sites')
		make_full_binding_site_representative_cluster(cluster,above_bridge_binding_sites,self.name_without_suffix+'_above_bridge_binding_sites')
		make_full_binding_site_representative_cluster(cluster,above_three_fold_sites,self.name_without_suffix    +'_above_three_fold_sites')
		make_full_binding_site_representative_cluster(cluster,above_four_fold_sites,self.name_without_suffix     +'_above_four_fold_sites')
		return above_atom_binding_sites, above_bridge_binding_sites, above_three_fold_sites, above_four_fold_sites

	def adsorb_species_to_cluster(self,cluster,binding_site_data,adsorbed_species,main_path_name):
		for an_adsorbed_species in adsorbed_species:
			for index in range(len(binding_site_data)):
				binding_site_datum = binding_site_data[index]
				adsorb_single_species_to_cluster(cluster, binding_site_datum, an_adsorbed_species, main_path_name, index+1)

	# ------------------------------------------------------------------------------------------------------------------------------------------------------



import os, shutil
import numpy as np

def make_new_folder(folder_path):
	if os.path.exists(folder_path):
		shutil.rmtree(folder_path)
	os.makedirs(folder_path)

def remove_folder(folder_path):
	if os.path.exists(folder_path):
		shutil.rmtree(folder_path)

def make_folder(folder_path):
	if not os.path.exists(folder_path):
		os.makedirs(folder_path)

def get_distance(atom_position_1,atom_position_2):
	xx_dist = atom_position_1[0] - atom_position_2[0]
	yy_dist = atom_position_1[1] - atom_position_2[1]
	zz_dist = atom_position_1[2] - atom_position_2[2]
	distance = (xx_dist**2.0 + yy_dist**2.0 + zz_dist**2.0)**0.5
	return distance

def get_unit_vector(v):
	v_hat = v / np.linalg.norm(v)
	return v_hat

def Rodrigues_Rotation_formula(vector, vector_to_rotate_onto):
	# https://en.wikipedia.org/wiki/Rodrigues%27_rotation_formula
	# https://math.stackexchange.com/questions/2828802/angle-definition-confusion-in-rodrigues-rotation-matrix

	theta = np.arccos(np.dot(vector, vector_to_rotate_onto) / np.linalg.norm(vector)*np.linalg.norm(vector_to_rotate_onto))
	k = np.cross(vector, vector_to_rotate_onto) / np.linalg.norm(np.cross(vector, vector_to_rotate_onto))

	K = np.array([[0, -k[2], k[1]],[k[2], 0, -k[0]],[-k[1], k[0], 0]])
	K2 = np.matmul(K,K)
	I = np.eye(3)

	R = I + np.sin(theta)*K + (1-np.cos(theta))*K2

	return R


import numpy as np

from ase import Atom
from ase.io import write

from Adsorber.Adsorber.other_methods import make_folder, get_unit_vector, Rodrigues_Rotation_formula

def get_extended_adsorption_site(distance_of_adatom_from_surface, adsorption_site, centre_of_binding_origin):
	vector = get_unit_vector(adsorption_site - centre_of_binding_origin)
	new_adsorption_site = centre_of_binding_origin + distance_of_adatom_from_surface*vector
	return new_adsorption_site

def return_vector(vector):
	if isinstance(vector,tuple) or isinstance(vector,list):
		return vector
	elif isinstance(vector,str):
		if vector == 'x':
			vector = (1,0,0)
		elif vector == 'y':
			vector = (0,1,0)
		elif vector == 'z':
			vector = (0,0,1)
		elif vector == '-x':
			vector = (-1,0,0)
		elif vector == '-y':
			vector = (0,-1,0)
		elif vector == '-z':
			vector = (0,0,-1)
		else:
			exit('error')
		return vector

def rotate_molecule_into_new_vector(molecule, molecule_vector, rotate_molecule_to_this_vector, adsorbed_molecule_name,Rotation_matrix=None):
	if Rotation_matrix is None:
		Rotation_matrix = Rodrigues_Rotation_formula(molecule_vector, rotate_molecule_to_this_vector)
	new_rotated_positions = np.transpose(np.matmul(Rotation_matrix,np.transpose(molecule.get_positions())))
	molecule.set_positions(new_rotated_positions)

def adsorb_single_species_to_cluster(cluster, binding_site_datum, an_adsorbed_species, main_path_name, sub_name):

	cluster_with_adsorbed_molecule = cluster.copy()

	adsorption_site, centre_of_binding_origin = binding_site_datum
	distance_of_adatom_from_surface = an_adsorbed_species['distance_of_adatom_from_surface']
	adsorption_site = get_extended_adsorption_site(distance_of_adatom_from_surface,adsorption_site, centre_of_binding_origin)

	adsorbed_molecule_name = an_adsorbed_species['name']
	adsorbed_molecule = an_adsorbed_species['molecule'].copy()

	path_name = main_path_name+'/'+adsorbed_molecule_name
	cluster_with_adsorbed_molecule_name = adsorbed_molecule_name+'_site_'+str(sub_name)
	make_folder(path_name)

	if len(adsorbed_molecule) == 1:
		if isinstance(adsorbed_molecule,Atom):
			# move index atom to the binding position
			adsorbed_molecule.x = adsorption_site[0]
			adsorbed_molecule.y = adsorption_site[1]
			adsorbed_molecule.z = adsorption_site[2]
			cluster_with_adsorbed_molecule.append(np.transpose(adsorbed_molecule))
		else:
			# move index atom to the binding position
			adsorbed_molecule[0].x = adsorption_site[0]
			adsorbed_molecule[0].y = adsorption_site[1]
			adsorbed_molecule[0].z = adsorption_site[2]
			cluster_with_adsorbed_molecule += adsorbed_molecule
		write(path_name+'/'+cluster_with_adsorbed_molecule_name+'.xyz',cluster_with_adsorbed_molecule)
	elif len(adsorbed_molecule) > 1:
		if 'rotations' in an_adsorbed_species:
			rotations = an_adsorbed_species['rotations']
		else:
			rotations = [0.0]
		adsorbed_molecule = an_adsorbed_species['molecule'].copy()
		binding_atom_index = an_adsorbed_species['index']
		adsorbed_molecule_axis = get_unit_vector(return_vector(an_adsorbed_species['axis']))
		position_of_index_atom = adsorbed_molecule[binding_atom_index].position
		center_index_atom = np.array((0,0,0)) - position_of_index_atom
		direction_to_point_molecule_along = get_unit_vector(adsorption_site - centre_of_binding_origin)
		Rotation_matrix = Rodrigues_Rotation_formula(adsorbed_molecule_axis, direction_to_point_molecule_along)
		for rotation in rotations:
			cluster_with_adsorbed_molecule = cluster.copy()
			adsorbed_molecule = an_adsorbed_species['molecule'].copy()
			adsorbed_molecule.rotate(rotation,adsorbed_molecule_axis,adsorbed_molecule[binding_atom_index].position)
			#-----------------------------------------------------------------------------
			# rotate the molecule to allign correctly
			# 1. center the index_atom at (0,0,0)
			adsorbed_molecule.set_positions(adsorbed_molecule.get_positions() + center_index_atom)		
			# 2. determine the angles required to rotate the molecule based on adsorbed_molecule_axis
			# 3. Perform rotation
			rotate_molecule_into_new_vector(adsorbed_molecule, adsorbed_molecule_axis, direction_to_point_molecule_along, adsorbed_molecule_name, Rotation_matrix=Rotation_matrix)
			#-----------------------------------------------------------------------------
			# move index atom to the binding position
			position_of_index_atom = adsorbed_molecule[binding_atom_index].position
			adsorbed_atom_translation = adsorption_site - position_of_index_atom
			adsorbed_molecule.set_positions(adsorbed_molecule.get_positions() + adsorbed_atom_translation)
			#-----------------------------------------------------------------------------
			cluster_with_adsorbed_molecule += adsorbed_molecule
			if len(rotations) == 1:
				write(path_name+'/'+cluster_with_adsorbed_molecule_name+'.xyz',cluster_with_adsorbed_molecule)
			else:
				write(path_name+'/'+cluster_with_adsorbed_molecule_name+'_rotation_'+str(rotation)+'.xyz',cluster_with_adsorbed_molecule)



from ase.io import read

def import_settings(self,name,cluster_or_surface_model,cutoff,surface_atoms,adsorbed_species,slurm_information):
	# General data about the cluster
	self.name = name
	self.cluster = read(name)
	import_cluster_or_surface_model_setting(self,cluster_or_surface_model)
	self.name_without_suffix = self.name.split('.')[0]
	# information about the surface of the cluster/surface model
	self.surface_atoms = sorted(surface_atoms)
	import_cutoff_setting(self,cutoff)
	# information about the atoms and molecules that will be adsorbed, as well sa their binding sites
	self.distance_of_adatom_from_surface = 1.25 # Angstroms #vdw_radii[atomic_numbers[self.cluster[0].symbol]] + vdw_radii[atomic_numbers['H']]
	self.adsorbed_species = adsorbed_species
	self.bind_site_data_types = ['above atom sites','above bridge sites','above three-fold sites','above four-fold sites']
	# information about where data from this program is stored
	self.data_storage_file = 'adsorber_data.txt'
	self.system_folder_name = 'System_with_Adsorbed_Species'
	# Information about making VASP file of system with adsorbed species
	self.vasp_files_folder = 'VASP_Files' 
	self.VASP_folder_name = 'System_with_Adsorbed_Species_for_VASP' 
	self.slurm_information = slurm_information

def import_cluster_or_surface_model_setting(self,cluster_or_surface_model):
	if not isinstance(cluster_or_surface_model,str):
		print('Error in Adsorber')
		print('The "cluster_or_surface_model" variable must be a string')
		print('"cluster_or_surface_model" = '+str(cluster_or_surface_model))
		print('Check this out. The Adsorber program will finish without completing')
		exit()
	elif cluster_or_surface_model.lower() in ['cluster','surface model']:
		self.cluster_or_surface_model = cluster_or_surface_model
	else:
		print('Error in Adsorber')
		print('The "cluster_or_surface_model" variable must be either "cluster" or "surface model"')
		print('"cluster_or_surface_model" = '+str(cluster_or_surface_model))
		print('Check this out. The Adsorber program will finish without completing')
		exit()

def import_cutoff_setting(self,cutoff):
	surface_atom_symbols = list(set([self.cluster[index].symbol for index in self.surface_atoms]))
	self.cutoff = {}
	if isinstance(cutoff,float):
		for element1 in surface_atom_symbols:
			for element2 in surface_atom_symbols:
				self.cutoff[(element1, element2)] = cutoff
	elif isinstance(cutoff,dict):
		issues = []
		for element in surface_atom_symbols:
			if not element in self.cutoff.keys():
				issues.append(element)
		for index1 in range(len(surface_atom_symbols)):
			element1 = surface_atom_symbols[index1]
			for index2 in range(index1+1,len(surface_atom_symbols)):
				element2 = surface_atom_symbols[index2]
				if (not (element1, element2) in self.cutoff.keys()) or (not (element2, element1) in self.cutoff.keys()):
					issues.append((element1, element2))
		if len(issues) == 0:
			for key, value in cutoff.items():
				if isinstance(key,str):
					self.cutoff[(key,key)] = value
				elif isinstance(key,tuple) and len(key) == 2:
					element1, element2 = key
					self.cutoff[(element1, element2)] = value
					self.cutoff[(element2, element1)] = value
		else:
			print('Error in Adsorber')
			print('You are missing the following from the "cutoff" variable dictionary:')
			print(str(issues))
			print('Make sure to add these cutoff values to your dictionary')
			print('Check this out. The Adsorber program will finish without completing')
			exit()
	else:
		print('Error in Adsorber')
		print('The "cutoff" variable must be either a float or a dictionary')
		print('"cutoff" = '+str(cutoff))
		print('"cutoff" is a '+str(type(cutoff)))
		print('Check this out. The Adsorber program will finish without completing')
		exit()		


import os
from setuptools import setup

def get_version_number():
  path_to_written_version = 'Adsorber/__init__.py'
  with open(path_to_written_version) as initPY:
    for line in initPY:
      if line.startswith('__version__'):
        version = eval(line.rstrip().replace('__version__ = ',''))
        break
  return version

def get_long_description():
  this_directory = os.path.abspath(os.path.dirname(__file__))
  readme_file = os.path.join(this_directory, 'README.md')
  with open(readme_file, encoding='utf-8') as f:
    long_description = f.read()
  return long_description

def find_packages(root):
  packages = []
  for root, dirs, files in os.walk(root, topdown=False):
    for name in files:
      if name.endswith('.py'):
        file = os.path.relpath(os.path.join(root, name))
        dirname = os.path.dirname(file)
        rel_dirname = os.path.relpath(dirname)
        if not rel_dirname in packages:
          packages.append(rel_dirname)
  return sorted(packages)

def find_scripts():
  scripts = []
  scripts_folders = []
  for scripts_folder in scripts_folders:
    for root, dirs, files in os.walk('Adsorber/'+scripts_folder, topdown=False):
      for file in files:
        if file.endswith('.py') and not 'Main' in file:
          filepath = os.path.relpath(os.path.join(root, file))
          scripts.append(filepath)
      dirs[:] = []
  return sorted(scripts)

setup(name='Adsorber',
      packages=find_packages(root='Adsorber'),
      scripts=find_scripts(),
      version=get_version_number(),
      description="This program is designed to adsorb molecules to the surface of a cluster in all various ways..",
      long_description=get_long_description(),
      long_description_content_type='text/markdown',
      author='Geoffrey R. Weal and Dr. Anna L. Garden',
      author_email='anna.garden@otago.ac.nz',
      url = 'https://blogs.otago.ac.nz/annagarden/',
      download_url = 'https://github.com/GardenGroupUO/Adsorber/archive/v'+str(get_version_number())+'.tar.gz',
      license='GNU AFFERO GENERAL PUBLIC LICENSE',
      zip_safe=False,
      keywords = ['Adsorber', 'ase', 'university-of-otago', 'atomic-simulation-environment', 'otago-university'],
      install_requires=[
          'numpy','scipy','matplotlib','ase>=3.19.0'
          ],
      classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Science/Research',      # Define that your audience are developers
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU Affero General Public License v3',   # Again, pick a license
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        ],
      )




import numpy as np

def write_data_file(data_storage_file,bind_site_data_types,above_atom_binding_sites,above_bridge_binding_sites,above_three_fold_sites,above_four_fold_sites):
    def write_to_datafileTXT(datafileTXT,binding_sites,description):
        counter = 1
        datafileTXT.write('The following sites are binding sites for '+str(description)+'.\n')
        datafileTXT.write('Site\t\t\tBinding Position\t\t\t\t\t\tSurface Position\n')
        for binding_position, surface_position in binding_sites:
            datafileTXT.write('Site '+str(counter)+'\t'+str(binding_position)+' \t'+str(surface_position)+'\n')
            counter += 1
        datafileTXT.write('--------------------------------------------------------------\n')
    with open(data_storage_file,'w') as datafileTXT:
        datafileTXT.write('--------------------------------------------------------------\n')
        datafileTXT.write('--------------------------------------------------------------\n')
        datafileTXT.write('The following data gives the positions of possible binding sites for adatoms and admolecules across a cluster or surface.\n\n')
        write_to_datafileTXT(datafileTXT,above_atom_binding_sites,bind_site_data_types[0])
        write_to_datafileTXT(datafileTXT,above_bridge_binding_sites,bind_site_data_types[1])
        write_to_datafileTXT(datafileTXT,above_three_fold_sites,bind_site_data_types[2])
        write_to_datafileTXT(datafileTXT,above_four_fold_sites,bind_site_data_types[3])

def load_data_file(data_storage_file,bind_site_data_types):
    index = 0
    write_to_list = False
    data = [[],[],[],[]]
    with open(data_storage_file,'r') as datafileTXT:
        for line in datafileTXT:
            if line.startswith('Site\t\t\tBinding Position\t\t\t\t\t\tSurface Position'):
                continue
            elif line.startswith('The following sites are binding sites for '+str(bind_site_data_types[index])+'.'):
                write_to_list = True
            elif line.startswith('--------------------------------------------------------------') and write_to_list == True:
                write_to_list = False
                index += 1
            elif write_to_list:
                line = line.rstrip()
                index2 = 0
                datum = ['','']
                write_to_second_list = False
                for character in line:
                    if character == '[':
                        write_to_second_list = True
                    elif character == ']':
                        datum[index2] = datum[index2].split()
                        for index_sub in range(len(datum[index2])):
                            datum[index2][index_sub] = float(datum[index2][index_sub])
                        write_to_second_list = False
                        index2 += 1
                    elif write_to_second_list:
                        datum[index2] += character
                binding_position, surface_position = datum
                binding_position = np.array(binding_position)
                surface_position = np.array(surface_position)
                datum = (binding_position,surface_position)
                data[index].append(datum)
    above_atom_binding_sites, above_bridge_binding_sites, above_three_fold_sites, above_four_fold_sites = data
    return above_atom_binding_sites, above_bridge_binding_sites, above_three_fold_sites, above_four_fold_sites


#!/usr/bin/env python3

import os
from shutil import copyfile
from ase.io import read, write

def make_VASP_folders(system,adsorbed_species,look_through_folder='System_with_Adsorbed_Species',vasp_files_folder='VASP_Files',folder_name='VASP_for_System_with_Adsorbed_Species',slurm_information={}):
	"""

	"""
	elements = get_elements(system,adsorbed_species)
	check_VASP_files(vasp_files_folder,elements)
	if not os.path.exists(folder_name):
		os.makedirs(folder_name)
	for root, dirs, files in os.walk(look_through_folder):
		dirs.sort()
		files.sort()
		for file in files:
			if file.endswith('.xyz'):
				print(root)
				dirs[:] = []
				break
		for file in files:
			if file.endswith('.xyz'):
				system = read(root+'/'+file)
				file_name = file.replace('.xyz','')
				folder_to_save_to = folder_name+root.replace(look_through_folder,'')+'/'+file_name
				if not os.path.exists(folder_to_save_to):
					os.makedirs(folder_to_save_to)
				write(folder_to_save_to+'/POSCAR',system)
				for vasp_file in os.listdir(vasp_files_folder):
					if not vasp_file == 'POTCARs':
						copyfile(vasp_files_folder+'/'+vasp_file,folder_to_save_to+'/'+vasp_file)
				make_overall_potcar(folder_to_save_to,vasp_files_folder)
				make_individual_submitSL_files(folder_to_save_to,file_name,slurm_information)

def get_elements(system,adsorbed_species):
	elements = []
	elements += list(system.get_chemical_symbols())
	for molecule_dict in adsorbed_species:
		elements += list(molecule_dict['molecule'].get_chemical_symbols())
	elements = tuple(set(elements))
	return elements

def make_overall_potcar(folder_to_save_to,vasp_files_folder):
	with open(folder_to_save_to+'/POSCAR') as POSCAR_file:
		elements = POSCAR_file.readline().split()
	with open(folder_to_save_to+'/POTCAR', "w") as out_POTCAR:
		for element in elements:
			with open(vasp_files_folder+'/POTCARs/POTCAR_'+str(element), "r") as in_POTCAR:
				out_POTCAR.write(in_POTCAR.read())

def check_VASP_files(vasp_files_folder,elements):
	"""

	"""
	if not os.path.exists(vasp_files_folder):
		print('Error in copying VASP files to cluster folder')
		print('There is no folder called "VASP_Files" in your working directory.')
		print('This folder is where you should place your VASP files in for DFT local optimisations.')
		print('Make this folder and place your "POTCAR" files for each element in your system and all adsorbed species, INCAR", "KPOINTS" files for VASP local optimisations')
		print('This program will exit without completing.')
		exit()
	VASP_Files_files = os.listdir(vasp_files_folder)
	have_INCAR   = 'INCAR'   in VASP_Files_files
	have_KPOINTS = 'KPOINTS' in VASP_Files_files
	if not (have_INCAR and have_KPOINTS):
		print('Error in copying VASP files to cluster folder')
		print('You need the following files when you are performing VASP calculations:')
		print()
		print('\tINCAR  :\t'  +('You have this' if have_INCAR   else 'You do not have this'))
		print('\tKPOINTS:\t'+('You have this' if have_KPOINTS else 'You do not have this'))
		print()
		print('Check this out. This program will now end without completing.')
		exit()
	potcar_folder_name = 'POTCARs'
	have_POTCARs = (potcar_folder_name in VASP_Files_files) and os.path.isdir(vasp_files_folder+'/'+potcar_folder_name)
	if not have_POTCARs:
		print('Error in copying VASP files to cluster folder')
		print('You need the following folders when you are performing VASP calculations:')
		print()
		print('\tPOTCARs:\t' +('You have this' if have_POTCARs  else 'You do not have this'))
		print()
		print('Check this out. This program will now end without completing.')
		exit()
	not_got_elements = []
	for element in elements:
		if not os.path.exists(vasp_files_folder+'/'+potcar_folder_name+'/'+'POTCAR_'+str(element)):
			not_got_elements.append(element)
	if not len(not_got_elements) == 0:
		print('Error in copying VASP files to cluster folder')
		print('You need to include the following POTCAR files in '+str(potcar_folder_name)+' to perform VASP calculations:')
		print()
		for element in not_got_elements:
			print('POTCAR_'+str(element))
		print()
		print('Check this out. This program will now end without completing.')
		exit()

def make_individual_submitSL_files(root,file_name,slurm_information):
	"""

	"""
	project = slurm_information['project']
	time = slurm_information['time']
	nodes = slurm_information['nodes']
	ntasks_per_node = slurm_information['ntasks_per_node']
	mem_per_cpu = slurm_information['mem-per-cpu']
	partition = slurm_information['partition']
	email = slurm_information['email']
	vasp_version = slurm_information['vasp_version']
	vasp_execution = slurm_information['vasp_execution']
	make_submitSL(file_name,root,project,time,nodes,ntasks_per_node,mem_per_cpu,partition=partition,email=email,vasp_version=vasp_version,vasp_execution=vasp_execution)

def make_submitSL(file_name,local_path,project,time,nodes,ntasks_per_node,mem_per_cpu,partition='large',email='',vasp_version='VASP/5.4.4-intel-2017a',vasp_execution='vasp_std'):
    # create name for job
    #print("creating submit.sl for "+str(file_name))
    name = 'Adsorber_Run_'+str(file_name)
    # writing the submit.sl script
    with open(local_path+"/submit.sl", "w") as submitSL:
        submitSL.write('#!/bin/bash -e\n')
        submitSL.write('#SBATCH -J ' + str(name) + '\n')
        submitSL.write('#SBATCH -A ' + str(project) + '         # Project Account\n')
        submitSL.write('#SBATCH --partition ' + str(partition) + '\n')
        submitSL.write('\n')
        submitSL.write('#SBATCH --time=' + str(time) + '     # Walltime\n')
        submitSL.write('#SBATCH --nodes=' + str(nodes) + '\n')
        submitSL.write('#On VASP, Ben Roberts recommends using the same number\n')
        submitSL.write('#of tasks on all nodes, even if this makes scheduling\n')
        submitSL.write('#a little more difficult\n')
        submitSL.write('#SBATCH --ntasks-per-node=' + str(ntasks_per_node) + '\n')
        submitSL.write('#SBATCH --mem-per-cpu=' + str(mem_per_cpu) + '\n')
        submitSL.write('\n')
        #submitSL.write("#SBATCH --hint=nomultithread    # don't use hyperthreading"+'\n')
        submitSL.write('#SBATCH --output=slurm-%j.out      # %x and %j are replaced by job name and ID'+'\n')
        submitSL.write('#SBATCH --error=slurm-%j.err'+'\n')
        if not email == '':
            submitSL.write('#SBATCH --mail-user=' + str(email) + '\n')
            submitSL.write('#SBATCH --mail-type=ALL\n')
        submitSL.write('#SBATCH --hint nomultithread\n')
        submitSL.write('\n')
        submitSL.write('module load '+str(vasp_version)+'\n')
        submitSL.write('srun -K '+str(vasp_execution)+'\n')


# The information about the Adsorber program

__name__    = 'The Adsorber Program'
__version__ = '1.1'
__author__  = 'Geoffrey Weal and Dr. Anna Garden'

import sys
if sys.version_info[0] == 2:
	toString = ''
	toString += '\n'
	toString += '================================================'+'\n'
	toString += 'This is the Adsorber Program'+'\n'
	toString += 'Version: '+str(__version__)+'\n'
	toString += '\n'
	toString += 'The Adsorber program requires Python3. You are attempting to execute this program in Python2.'+'\n'
	toString += 'Make sure you are running the Adsorber program in Python3 and try again'+'\n'
	toString += 'This program will exit before beginning'+'\n'
	toString += '================================================'+'\n'
	raise ImportError(toString)
if sys.version_info[1] < 4:
	toString = ''
	toString += '\n'
	toString += '================================================'+'\n'
	toString += 'This is the Adsorber Program'+'\n'
	toString += 'Version: '+str(__version__)+'\n'
	toString += '\n'
	toString += 'The Adsorber program requires Python 3.4 or greater.'+'\n'
	toString += 'You are using Python '+str('.'.join(sys.version_info))
	toString += '\n'
	toString += 'Use a version of Python 3 that is greater or equal to Python 3.4.\n'
	toString += 'This program will exit before beginning'+'\n'
	toString += '================================================'+'\n'
	raise ImportError(toString)

# ------------------------------------------------------------------------------------------------------------------------

# A check for ASE
import importlib
python_version = 3.4

ase_spec = importlib.util.find_spec("ase")
found = ase_spec is not None

if not found:
	toString = ''
	toString += '\n'
	toString += '================================================'+'\n'
	toString += 'This is the Adsorber Program'+'\n'
	toString += 'Version: '+str(__version__)+'\n'
	toString += '\n'
	toString += 'The Adsorber program requires ASE.'+'\n'
	toString += '\n'
	toString += 'Install ASE through pip by following the instruction in https://adsorber.readthedocs.io/en/latest/Installation.html'+'\n'
	toString += 'These instructions will ask you to install ase by typing the following into your terminal\n'
	toString += 'pip3 install --user --upgrade ase\n'
	toString += '\n'
	toString += 'This program will exit before beginning'+'\n'
	toString += '================================================'+'\n'
	raise ImportError(toString)	

import ase
ase_version_minimum = '3.19.0'
from packaging import version
#from distutils.version import StrictVersion
#if StrictVersion(ase.__version__) < StrictVersion(ase_version_minimum):
if version.parse(ase.__version__) < version.parse(ase_version_minimum):
	toString = ''
	toString += '\n'
	toString += '================================================'+'\n'
	toString += 'This is the Adsorber Program'+'\n'
	toString += 'Version: '+str(__version__)+'\n'
	toString += '\n'
	toString += 'The Adsorber program requires ASE greater than or equal to '+str(ase_version_minimum)+'.'+'\n'
	toString += 'The current version of ASE you are using is '+str(ase.__version__)+'.'+'\n'
	toString += '\n'
	toString += 'Install ASE through pip by following the instruction in https://adsorber.readthedocs.io/en/latest/Installation.html'+'\n'
	toString += 'These instructions will ask you to install ase by typing the following into your terminal\n'
	toString += 'pip3 install --user --upgrade ase\n'
	toString += '\n'
	toString += 'This program will exit before beginning'+'\n'
	toString += '================================================'+'\n'
	raise ImportError(toString)

# ------------------------------------------------------------------------------------------------------------------------

__author_email__ = 'anna.garden@otago.ac.nz'
__license__ = 'GNU AFFERO GENERAL PUBLIC LICENSE'
__url__ = 'https://github.com/GardenGroupUO/Adsorber'
__doc__ = 'See https://adsorber.readthedocs.io/en/latest/ for the documentation on this program'

from Adsorber.Adsorber.Adsorber_Program import Adsorber_Program
__all__ = ['Adsorber_Program'] 

# ------------------------------------------------------------------------------------------------------------------------

